@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2019-2021 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::ops::Deref;
@@ -22,6 +22,7 @@ pub struct Id(pub Uuid);
 impl Deref for Id {
   type Target = Uuid;
 
+  #[inline]
   fn deref(&self) -> &Self::Target {
     &self.0
   }
@@ -152,6 +153,7 @@ Endpoint! {
   ],
   Err => GetError, []
 
+  #[inline]
   fn path(_input: &Self::Input) -> Str {
     "/v2/account".into()
   }
@@ -202,12 +204,12 @@ mod tests {
 }"#;
 
     let id = Id(Uuid::parse_str("904837e3-3b76-47ec-b432-046db621571b").unwrap());
-    let acc = from_json::<Account>(&response).unwrap();
+    let acc = from_json::<Account>(response).unwrap();
     assert_eq!(acc.id, id);
     assert_eq!(acc.status, Status::Active);
     assert_eq!(acc.currency, "USD");
     assert_eq!(acc.buying_power, Num::from(0));
-    assert_eq!(acc.trading_blocked, false);
+    assert!(!acc.trading_blocked);
     assert_eq!(
       acc.created_at,
       DateTime::parse_from_rfc3339("2018-10-01T13:35:25Z").unwrap()
