@@ -1,4 +1,4 @@
-// Copyright (C) 2020-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2020-2021 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use http::Method;
@@ -50,6 +50,7 @@ Endpoint! {
   ],
   Err => GetError, []
 
+  #[inline]
   fn path(_input: &Self::Input) -> Str {
     "/v2/account/configurations".into()
   }
@@ -69,10 +70,12 @@ Endpoint! {
     /* 400 */ BAD_REQUEST => InvalidValues,
   ]
 
+  #[inline]
   fn method() -> Method {
     Method::PATCH
   }
 
+  #[inline]
   fn path(_input: &Self::Input) -> Str {
     "/v2/account/configurations".into()
   }
@@ -106,7 +109,7 @@ mod tests {
   "trade_confirm_email": "all"
 }"#;
 
-    let config = from_json::<Configuration>(&response).unwrap();
+    let config = from_json::<Configuration>(response).unwrap();
     assert_eq!(config.trade_confirmation, TradeConfirmation::Email);
     assert!(!config.trading_suspended);
     assert!(!config.no_shorting);
