@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2019-2021 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::fmt::Display;
@@ -24,6 +24,7 @@ pub struct Id(pub Uuid);
 impl Deref for Id {
   type Target = Uuid;
 
+  #[inline]
   fn deref(&self) -> &Self::Target {
     &self.0
   }
@@ -39,6 +40,7 @@ pub enum Class {
 }
 
 impl AsRef<str> for Class {
+  #[inline]
   fn as_ref(&self) -> &'static str {
     match *self {
       Class::UsEquity => "us_equity",
@@ -47,6 +49,7 @@ impl AsRef<str> for Class {
 }
 
 impl Default for Class {
+  #[inline]
   fn default() -> Self {
     Self::UsEquity
   }
@@ -77,6 +80,7 @@ pub enum Status {
 }
 
 impl AsRef<str> for Status {
+  #[inline]
   fn as_ref(&self) -> &'static str {
     match *self {
       Status::Active => "active",
@@ -86,6 +90,7 @@ impl AsRef<str> for Status {
 }
 
 impl Default for Status {
+  #[inline]
   fn default() -> Self {
     Self::Active
   }
@@ -136,6 +141,7 @@ pub enum Symbol {
 }
 
 impl From<Id> for Symbol {
+  #[inline]
   fn from(symbol: Id) -> Self {
     Self::Id(symbol)
   }
@@ -145,6 +151,7 @@ impl<S> From<S> for Symbol
 where
   S: Into<String>,
 {
+  #[inline]
   fn from(symbol: S) -> Self {
     Self::Sym(symbol.into())
   }
@@ -193,7 +200,7 @@ impl FromStr for Symbol {
 impl Display for Symbol {
   fn fmt(&self, fmt: &mut Formatter<'_>) -> FmtResult {
     match self {
-      Self::Sym(sym) => fmt.write_str(&sym),
+      Self::Sym(sym) => fmt.write_str(sym),
       Self::SymExchg(sym, exchg) => write!(fmt, "{}:{}", sym, exchg.as_ref()),
       Self::SymExchgCls(sym, exchg, cls) => {
         write!(fmt, "{}:{}:{}", sym, exchg.as_ref(), cls.as_ref())
@@ -333,6 +340,7 @@ Endpoint! {
     /* 404 */ NOT_FOUND => NotFound,
   ]
 
+  #[inline]
   fn path(input: &Self::Input) -> Str {
     format!("/v2/assets/{}", input).into()
   }
@@ -427,16 +435,16 @@ mod tests {
 }"#;
 
     let id = Id(Uuid::parse_str("904837e3-3b76-47ec-b432-046db621571b").unwrap());
-    let asset = from_json::<Asset>(&response).unwrap();
+    let asset = from_json::<Asset>(response).unwrap();
     assert_eq!(asset.id, id);
     assert_eq!(asset.class, Class::UsEquity);
     assert_eq!(asset.exchange, Exchange::Nasdaq);
     assert_eq!(asset.symbol, "AAPL");
     assert_eq!(asset.status, Status::Active);
-    assert_eq!(asset.tradable, true);
-    assert_eq!(asset.marginable, true);
-    assert_eq!(asset.shortable, true);
-    assert_eq!(asset.easy_to_borrow, true);
+    assert!(asset.tradable);
+    assert!(asset.marginable);
+    assert!(asset.shortable);
+    assert!(asset.easy_to_borrow);
   }
 
   #[test]
@@ -454,7 +462,7 @@ mod tests {
   "fractionable": true
 }"#;
 
-    let asset = from_json::<Asset>(&response).unwrap();
+    let asset = from_json::<Asset>(response).unwrap();
     assert_eq!(asset.exchange, Exchange::Unknown);
   }
 
@@ -472,7 +480,7 @@ mod tests {
       assert_eq!(asset.exchange, Exchange::Nasdaq);
       assert_eq!(asset.symbol, "AAPL");
       assert_eq!(asset.status, Status::Active);
-      assert_eq!(asset.tradable, true);
+      assert!(asset.tradable);
     }
 
     let symbols = [
