@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2019-2021 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use chrono::DateTime;
@@ -37,6 +37,7 @@ Endpoint! {
   ],
   Err => GetError, []
 
+  #[inline]
   fn path(_input: &Self::Input) -> Str {
     "/v2/clock".into()
   }
@@ -66,8 +67,8 @@ mod tests {
   "next_close": "2018-04-01T12:00:00.000Z"
 }"#;
 
-    let clock = from_json::<Clock>(&response).unwrap();
-    assert_eq!(clock.open, true);
+    let clock = from_json::<Clock>(response).unwrap();
+    assert!(clock.open);
   }
 
   #[test(tokio::test)]
