@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2019-2021 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 /// Definitions pertaining the user's account.
@@ -23,6 +23,10 @@ pub mod orders;
 pub mod position;
 /// Functionality for listing open positions.
 pub mod positions;
+/// Definitions surrounding watchlists.
+pub mod watchlist;
+/// Functionality for listing watchlists.
+pub mod watchlists;
 
 mod de;
 mod util;
