@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2019-2021 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::ops::Deref;
@@ -24,8 +24,6 @@ use serde_urlencoded::to_string as to_query;
 use uuid::Uuid;
 
 use crate::api::v2::asset;
-use crate::api::v2::util::u64_from_str;
-use crate::api::v2::util::u64_to_str;
 use crate::Str;
 
 
@@ -36,6 +34,7 @@ pub struct Id(pub Uuid);
 impl Deref for Id {
   type Target = Uuid;
 
+  #[inline]
   fn deref(&self) -> &Self::Target {
     &self.0
   }
@@ -139,6 +138,7 @@ pub enum Side {
 impl Not for Side {
   type Output = Self;
 
+  #[inline]
   fn not(self) -> Self::Output {
     match self {
       Self::Buy => Self::Sell,
@@ -174,6 +174,7 @@ pub enum Class {
 }
 
 impl Default for Class {
+  #[inline]
   fn default() -> Self {
     Self::Simple
   }
@@ -202,6 +203,7 @@ pub enum Type {
 }
 
 impl Default for Type {
+  #[inline]
   fn default() -> Self {
     Self::Market
   }
@@ -229,6 +231,7 @@ pub enum TimeInForce {
 }
 
 impl Default for TimeInForce {
+  #[inline]
   fn default() -> Self {
     Self::Day
   }
@@ -291,6 +294,45 @@ impl Serialize for StopLoss {
 }
 
 
+/// An abstraction to be able to handle orders in both notional and quantity units.
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[serde(untagged)]
+pub enum Amount {
+  /// Wrapper for the quantity field.
+  Quantity {
+    /// A number of shares to order. This can be a fractional number if
+    /// trading fractionals or a whole number if not.
+    #[serde(rename = "qty")]
+    quantity: Num,
+  },
+  /// Wrapper for the notional field.
+  Notional {
+    /// A dollar amount to use for the order. This can result in
+    /// fractional quantities.
+    #[serde(rename = "notional")]
+    notional: Num,
+  },
+}
+
+impl Amount {
+  /// Helper method to initialize a quantity.
+  #[inline]
+  pub fn quantity(amount: impl Into<Num>) -> Self {
+    Self::Quantity {
+      quantity: amount.into(),
+    }
+  }
+
+  /// Helper method to initialize a notional.
+  #[inline]
+  pub fn notional(amount: impl Into<Num>) -> Self {
+    Self::Notional {
+      notional: amount.into(),
+    }
+  }
+}
+
+
 /// A helper for initializing `OrderReq` objects.
 #[derive(Clone, Debug, Default, PartialEq)]
 pub struct OrderReqInit {
@@ -322,13 +364,13 @@ pub struct OrderReqInit {
 
 impl OrderReqInit {
   /// Create an `OrderReq` from an `OrderReqInit`.
-  pub fn init<S>(self, symbol: S, side: Side, quantity: u64) -> OrderReq
+  pub fn init<S>(self, symbol: S, side: Side, amount: Amount) -> OrderReq
   where
     S: Into<asset::Symbol>,
   {
     OrderReq {
       symbol: symbol.into(),
-      quantity,
+      amount,
       side,
       class: self.class,
       type_: self.type_,
@@ -352,9 +394,9 @@ pub struct OrderReq {
   /// Symbol or asset ID to identify the asset to trade.
   #[serde(rename = "symbol")]
   pub symbol: asset::Symbol,
-  /// Number of shares to trade.
-  #[serde(rename = "qty", serialize_with = "u64_to_str")]
-  pub quantity: u64,
+  /// Amount of shares to trade.
+  #[serde(flatten)]
+  pub amount: Amount,
   /// The side the order is on.
   #[serde(rename = "side")]
   pub side: Side,
@@ -407,7 +449,7 @@ pub struct OrderReq {
 #[derive(Clone, Debug, Default, PartialEq)]
 pub struct ChangeReqInit {
   /// See `ChangeReq::quantity`.
-  pub quantity: u64,
+  pub quantity: Num,
   /// See `ChangeReq::time_in_force`.
   pub time_in_force: TimeInForce,
   /// See `ChangeReq::limit_price`.
@@ -438,8 +480,8 @@ impl ChangeReqInit {
 #[derive(Clone, Debug, Serialize, PartialEq)]
 pub struct ChangeReq {
   /// Number of shares to trade.
-  #[serde(rename = "qty", serialize_with = "u64_to_str")]
-  pub quantity: u64,
+  #[serde(rename = "qty")]
+  pub quantity: Num,
   /// How long the order will be valid.
   #[serde(rename = "time_in_force")]
   pub time_in_force: TimeInForce,
@@ -507,20 +549,12 @@ pub struct Order {
   /// The symbol of the asset being traded.
   #[serde(rename = "symbol")]
   pub symbol: String,
-  /// The quantity being requested.
-  #[serde(
-    rename = "qty",
-    deserialize_with = "u64_from_str",
-    serialize_with = "u64_to_str"
-  )]
-  pub quantity: u64,
+  /// The amount being requested.
+  #[serde(flatten)]
+  pub amount: Amount,
   /// The quantity that was filled.
-  #[serde(
-    rename = "filled_qty",
-    deserialize_with = "u64_from_str",
-    serialize_with = "u64_to_str"
-  )]
-  pub filled_quantity: u64,
+  #[serde(rename = "filled_qty")]
+  pub filled_quantity: Num,
   /// The type of order.
   #[serde(rename = "type")]
   pub type_: Type,
@@ -593,6 +627,7 @@ Endpoint! {
     /* 404 */ NOT_FOUND => NotFound,
   ]
 
+  #[inline]
   fn path(_input: &Self::Input) -> Str {
     "/v2/orders:by_client_order_id".into()
   }
@@ -605,7 +640,7 @@ Endpoint! {
     }
 
     let order_id = ClientOrderId {
-      order_id: &input,
+      order_id: input,
     };
     Ok(Some(to_query(order_id)?.into()))
   }
@@ -626,10 +661,12 @@ Endpoint! {
     /* 422 */ UNPROCESSABLE_ENTITY => InvalidInput,
   ]
 
+  #[inline]
   fn method() -> Method {
     Method::POST
   }
 
+  #[inline]
   fn path(_input: &Self::Input) -> Str {
     "/v2/orders".into()
   }
@@ -659,6 +696,7 @@ Endpoint! {
     /* 422 */ UNPROCESSABLE_ENTITY => InvalidInput,
   ]
 
+  #[inline]
   fn method() -> Method {
     Method::PATCH
   }
@@ -692,6 +730,7 @@ EndpointNoParse! {
     /* 422 */ UNPROCESSABLE_ENTITY => NotCancelable,
   ]
 
+  #[inline]
   fn method() -> Method {
     Method::DELETE
   }
@@ -700,6 +739,7 @@ EndpointNoParse! {
     format!("/v2/orders/{}", input.to_simple()).into()
   }
 
+  #[inline]
   fn parse(body: &[u8]) -> Result<Self::Output, Self::ConversionError> {
     debug_assert_eq!(body, b"");
     Ok(())
@@ -715,6 +755,8 @@ EndpointNoParse! {
 mod tests {
   use super::*;
 
+  use std::str::FromStr as _;
+
   use futures::TryFutureExt;
 
   use serde_json::from_str as from_json;
@@ -766,6 +808,26 @@ mod tests {
     assert_eq!(to_json(&stop_loss).unwrap(), &expected[..]);
   }
 
+  /// Check that we can parse the `Amount::quantity` variant properly.
+  #[test]
+  fn parse_quantity_amount() {
+    let serialized = r#"{
+    "qty": "15"
+}"#;
+    let amount = from_json::<Amount>(serialized).unwrap();
+    assert_eq!(amount, Amount::quantity(15));
+  }
+
+  /// Check that we can parse the `Amount::notional` variant properly.
+  #[test]
+  fn parse_notional_amount() {
+    let serialized = r#"{
+    "notional": "15.12"
+}"#;
+    let amount = from_json::<Amount>(serialized).unwrap();
+    assert_eq!(amount, Amount::notional(Num::from_str("15.12").unwrap()));
+  }
+
   #[test]
   fn parse_reference_order() {
     let response = r#"{
@@ -795,14 +857,14 @@ mod tests {
 }"#;
 
     let id = Id(Uuid::parse_str("904837e3-3b76-47ec-b432-046db621571b").unwrap());
-    let order = from_json::<Order>(&response).unwrap();
+    let order = from_json::<Order>(response).unwrap();
     assert_eq!(order.id, id);
     assert_eq!(
       order.created_at,
       DateTime::parse_from_rfc3339("2018-10-05T05:48:59Z").unwrap()
     );
     assert_eq!(order.symbol, "AAPL");
-    assert_eq!(order.quantity, 15);
+    assert_eq!(order.amount, Amount::quantity(15));
     assert_eq!(order.type_, Type::Market);
     assert_eq!(order.time_in_force, TimeInForce::Day);
     assert_eq!(order.limit_price, Some(Num::from(107)));
@@ -820,7 +882,7 @@ mod tests {
         extended_hours,
         ..Default::default()
       }
-      .init(symbol, Side::Buy, 1);
+      .init(symbol, Side::Buy, Amount::quantity(1));
 
       let api_info = ApiInfo::from_env().unwrap();
       let client = Client::new(api_info);
@@ -829,7 +891,7 @@ mod tests {
       client.issue::<Delete>(&order.id).await.unwrap();
 
       assert_eq!(order.symbol, "SPY");
-      assert_eq!(order.quantity, 1);
+      assert_eq!(order.amount, Amount::quantity(1));
       assert_eq!(order.side, Side::Buy);
       assert_eq!(order.type_, Type::Limit);
       assert_eq!(order.time_in_force, TimeInForce::Day);
@@ -863,7 +925,7 @@ mod tests {
       trail_price: Some(Num::from(50)),
       ..Default::default()
     }
-    .init(symbol, Side::Buy, 1);
+    .init(symbol, Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
@@ -872,7 +934,7 @@ mod tests {
     client.issue::<Delete>(&order.id).await.unwrap();
 
     assert_eq!(order.symbol, "SPY");
-    assert_eq!(order.quantity, 1);
+    assert_eq!(order.amount, Amount::quantity(1));
     assert_eq!(order.side, Side::Buy);
     assert_eq!(order.type_, Type::TrailingStop);
     assert_eq!(order.time_in_force, TimeInForce::Day);
@@ -892,7 +954,7 @@ mod tests {
       trail_percent: Some(Num::from(10)),
       ..Default::default()
     }
-    .init(symbol, Side::Buy, 1);
+    .init(symbol, Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
@@ -901,7 +963,7 @@ mod tests {
     client.issue::<Delete>(&order.id).await.unwrap();
 
     assert_eq!(order.symbol, "SPY");
-    assert_eq!(order.quantity, 1);
+    assert_eq!(order.amount, Amount::quantity(1));
     assert_eq!(order.side, Side::Buy);
     assert_eq!(order.type_, Type::TrailingStop);
     assert_eq!(order.time_in_force, TimeInForce::Day);
@@ -922,7 +984,7 @@ mod tests {
       stop_loss: Some(StopLoss::Stop(Num::from(1))),
       ..Default::default()
     }
-    .init("SPY", Side::Buy, 1);
+    .init("SPY", Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
@@ -935,7 +997,7 @@ mod tests {
     }
 
     assert_eq!(order.symbol, "SPY");
-    assert_eq!(order.quantity, 1);
+    assert_eq!(order.amount, Amount::quantity(1));
     assert_eq!(order.side, Side::Buy);
     assert_eq!(order.type_, Type::Limit);
     assert_eq!(order.time_in_force, TimeInForce::Day);
@@ -956,7 +1018,7 @@ mod tests {
       stop_loss: Some(StopLoss::Stop(Num::from(1))),
       ..Default::default()
     }
-    .init("SPY", Side::Buy, 1);
+    .init("SPY", Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
@@ -969,7 +1031,7 @@ mod tests {
     }
 
     assert_eq!(order.symbol, "SPY");
-    assert_eq!(order.quantity, 1);
+    assert_eq!(order.amount, Amount::quantity(1));
     assert_eq!(order.side, Side::Buy);
     assert_eq!(order.type_, Type::Limit);
     assert_eq!(order.time_in_force, TimeInForce::Day);
@@ -993,7 +1055,7 @@ mod tests {
         limit_price: Some(Num::from(1)),
         ..Default::default()
       }
-      .init("AAPL", Side::Buy, 1);
+      .init("AAPL", Side::Buy, Amount::quantity(1));
 
       match client.issue::<Post>(&request).await {
         Ok(order) => {
@@ -1022,7 +1084,43 @@ mod tests {
       limit_price: Some(Num::from(1000)),
       ..Default::default()
     }
-    .init("AAPL", Side::Buy, 100000);
+    .init("AAPL", Side::Buy, Amount::quantity(100_000));
+
+    let result = client.issue::<Post>(&request).await;
+    let err = result.unwrap_err();
+
+    match err {
+      RequestError::Endpoint(PostError::InsufficientFunds(..)) => (),
+      _ => panic!("Received unexpected error: {:?}", err),
+    };
+  }
+
+  /// Test that we can submit an order with a notional amount.
+  #[test(tokio::test)]
+  async fn submit_unsatisfiable_notional_order() {
+    let request =
+      OrderReqInit::default().init("SPY", Side::Buy, Amount::notional(Num::new(10_000_000, 3)));
+
+    let api_info = ApiInfo::from_env().unwrap();
+    let client = Client::new(api_info);
+
+    let result = client.issue::<Post>(&request).await;
+    let err = result.unwrap_err();
+
+    match err {
+      RequestError::Endpoint(PostError::InsufficientFunds(..)) => (),
+      _ => panic!("Received unexpected error: {:?}", err),
+    };
+  }
+
+  /// Test that we can submit an order with a fractional quantity.
+  #[test(tokio::test)]
+  async fn submit_unsatisfiable_fractional_order() {
+    let qty = Num::from(1_000_000) + Num::new(1, 2);
+    let request = OrderReqInit::default().init("SPY", Side::Buy, Amount::quantity(qty));
+
+    let api_info = ApiInfo::from_env().unwrap();
+    let client = Client::new(api_info);
 
     let result = client.issue::<Post>(&request).await;
     let err = result.unwrap_err();
@@ -1062,7 +1160,7 @@ mod tests {
     assert_eq!(posted.asset_class, gotten.asset_class);
     assert_eq!(posted.asset_id, gotten.asset_id);
     assert_eq!(posted.symbol, gotten.symbol);
-    assert_eq!(posted.quantity, gotten.quantity);
+    assert_eq!(posted.amount, gotten.amount);
     assert_eq!(posted.type_, gotten.type_);
     assert_eq!(posted.side, gotten.side);
     assert_eq!(posted.time_in_force, gotten.time_in_force);
@@ -1088,7 +1186,7 @@ mod tests {
       extended_hours: true,
       ..Default::default()
     }
-    .init("SPY", Side::Buy, 1);
+    .init("SPY", Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
@@ -1111,14 +1209,14 @@ mod tests {
       limit_price: Some(Num::from(1)),
       ..Default::default()
     }
-    .init("AAPL", Side::Buy, 1);
+    .init("AAPL", Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
     let order = client.issue::<Post>(&request).await.unwrap();
 
     let request = ChangeReqInit {
-      quantity: 2,
+      quantity: Num::from(2),
       time_in_force: TimeInForce::UntilCanceled,
       limit_price: Some(Num::from(2)),
       ..Default::default()
@@ -1136,7 +1234,7 @@ mod tests {
 
     match result {
       Ok(order) => {
-        assert_eq!(order.quantity, 2);
+        assert_eq!(order.amount, Amount::quantity(2));
         assert_eq!(order.time_in_force, TimeInForce::UntilCanceled);
         assert_eq!(order.limit_price, Some(Num::from(2)));
         assert_eq!(order.stop_price, None);
@@ -1159,7 +1257,7 @@ mod tests {
       trail_price: Some(Num::from(20)),
       ..Default::default()
     }
-    .init("SPY", Side::Buy, 1);
+    .init("SPY", Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
@@ -1203,7 +1301,7 @@ mod tests {
       client_order_id: Some(client_order_id.clone()),
       ..Default::default()
     }
-    .init("SPY", Side::Buy, 1);
+    .init("SPY", Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
