@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2019-2021 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::borrow::Cow;
@@ -48,6 +48,7 @@ pub struct Builder {
 
 impl Builder {
   /// Adjust the maximum number of idle connections per host.
+  #[inline]
   pub fn max_idle_per_host(&mut self, max_idle: usize) -> &mut Self {
     let _ = self.builder.pool_max_idle_per_host(max_idle);
     self
@@ -80,6 +81,7 @@ impl Default for Builder {
   }
 
   #[cfg(not(test))]
+  #[inline]
   fn default() -> Self {
     Self {
       builder: HttpClient::builder(),
@@ -98,12 +100,14 @@ pub struct Client {
 
 impl Client {
   /// Instantiate a new `Builder` which allows for creating a customized `Client`.
+  #[inline]
   pub fn builder() -> Builder {
     Builder::default()
   }
 
   /// Create a new `Client` using the given key ID and secret for
   /// connecting to the API.
+  #[inline]
   pub fn new(api_info: ApiInfo) -> Self {
     Builder::default().build(api_info)
   }
@@ -117,8 +121,8 @@ impl Client {
       .map(|url| Url::parse(url.as_ref()).expect("endpoint definition contains invalid URL"))
       .unwrap_or_else(|| self.api_info.base_url.clone());
 
-    url.set_path(&R::path(&input));
-    url.set_query(R::query(&input)?.as_ref().map(AsRef::as_ref));
+    url.set_path(&R::path(input));
+    url.set_query(R::query(input)?.as_ref().map(AsRef::as_ref));
 
     let request = HttpRequestBuilder::new()
       .method(R::method())
@@ -141,7 +145,7 @@ impl Client {
   where
     R: Endpoint,
   {
-    let result = self.request::<R>(&input);
+    let result = self.request::<R>(input);
     async move {
       let request = result.map_err(RequestError::Endpoint)?;
       let span = span!(
@@ -200,6 +204,7 @@ impl Client {
   }
 
   /// Retrieve the `ApiInfo` object used by this `Client` instance.
+  #[inline]
   pub fn api_info(&self) -> &ApiInfo {
     &self.api_info
   }
